import click
from click.testing import CliRunner
from omi.cli import translate
import json


def test_cli_translation():
    @click.command()
    @click.argument("name")
    def hello(name):
        click.echo("Hello %s!" % name)

    runner = CliRunner()
    result = runner.invoke(
        translate,
        ["-f", "oep-v1.3", "-t", "oep-v1.4", "-omit_nones", "True", "tests/data/metadata_v13.json"],
    )
    with open("tests/data/metadata_v13_converted.json") as expected_file:
        expected = json.loads(expected_file.read())
        print(result)
        assert result.exit_code == 0
        assert json.loads(result.output) == expected



import os
import json
from copy import deepcopy


"""
This script transforms OEMetadata v1.4.1 (oem141) into OEMetadata v.1.5.1 (oem151).
It does so by backfilling the key-values from oem141 files into an empty oem151 template.json.
Default keys and values from oem151 are provided, if keys are not used or values are not filled in oem141.
The method applies for all key-values, except for the ‘foreignKeys‘ key, which is a direct copy of the list in oem141
and the '_comment' and 'metaMetadata' key, which are not processed, as new oem151 information is provided

Files are read from ./JSON/v141
Files are saved to ./JSON/v151

"""

# create directories or pass if exist
oem_versions = ["v141", "v151"]
for oem in oem_versions:
    try:
        os.makedirs(f"./JSON/{oem}")
    except FileExistsError:
        print(f"The directory: ./JSON/{oem} exists already")
        pass

# load oem141 file paths
json_path = "./JSON/v141"
json_files = os.listdir(json_path)
all_json_files_paths = [json_path + "/" + json_file for json_file in json_files]

# oem151 template
oem151 = {
    "name": None,
    "title": None,
    "id": None,
    "description": None,
    "language": [None],
    "subject": [{"name": None, "path": None}],
    "keywords": [None],
    "publicationDate": None,
    "context": {
        "homepage": None,
        "documentation": None,
        "sourceCode": None,
        "contact": None,
        "grantNo": None,
        "fundingAgency": None,
        "fundingAgencyLogo": None,
        "publisherLogo": None,
    },
    "spatial": {"location": None, "extent": None, "resolution": None},
    "temporal": {
        "referenceDate": None,
        "timeseries": [
            {
                "start": None,
                "end": None,
                "resolution": None,
                "alignment": None,
                "aggregationType": None,
            },
            {
                "start": None,
                "end": None,
                "resolution": None,
                "alignment": None,
                "aggregationType": None,
            },
        ],
    },
    "sources": [
        {
            "title": None,
            "description": None,
            "path": None,
            "licenses": [
                {
                    "name": None,
                    "title": None,
                    "path": None,
                    "instruction": None,
                    "attribution": None,
                }
            ],
        },
        {
            "title": None,
            "description": None,
            "path": None,
            "licenses": [
                {
                    "name": None,
                    "title": None,
                    "path": None,
                    "instruction": None,
                    "attribution": None,
                }
            ],
        },
    ],
    "licenses": [
        {
            "name": None,
            "title": None,
            "path": None,
            "instruction": None,
            "attribution": None,
        }
    ],
    "contributors": [
        {"title": None, "email": None, "date": None, "object": None, "comment": None}
    ],
    "resources": [
        {
            "profile": None,
            "name": None,
            "path": None,
            "format": None,
            "encoding": None,
            "schema": {
                "fields": [
                    {
                        "name": None,
                        "description": None,
                        "type": None,
                        "unit": None,
                        "isAbout": [{"name": None, "path": None}],
                        "valueReference": [{"value": None, "name": None, "path": None}],
                    },
                    {
                        "name": None,
                        "description": None,
                        "type": None,
                        "unit": None,
                        "isAbout": [{"name": None, "path": None}],
                        "valueReference": [{"value": None, "name": None, "path": None}],
                    },
                ],
                "primaryKey": [None],
                "foreignKeys": [
                    {
                        "fields": [None],
                        "reference": {"resource": None, "fields": [None]},
                    }
                ],
            },
            "dialect": {"delimiter": None, "decimalSeparator": "."},
        }
    ],
    "@id": None,
    "@context": None,
    "review": {"path": None, "badge": None},
    "metaMetadata": {
        "metadataVersion": "OEP-1.5.0",
        "metadataLicense": {
            "name": "CC0-1.0",
            "title": "Creative Commons Zero v1.0 Universal",
            "path": "https://creativecommons.org/publicdomain/zero/1.0/",
        },
    },
    "_comment": {
        "metadata": "Metadata documentation and explanation (https://github.com/OpenEnergyPlatform/oemetadata)",
        "dates": "Dates and time must follow the ISO8601 including time zone (YYYY-MM-DD or YYYY-MM-DDThh:mm:ssÂ±hh)",
        "units": "Use a space between numbers and units (100 m)",
        "languages": "Languages must follow the IETF (BCP47) format (en-GB, en-US, de-DE)",
        "licenses": "License name must follow the SPDX License List (https://spdx.org/licenses/)",
        "review": "Following the OEP Data Review (https://github.com/OpenEnergyPlatform/data-preprocessing/blob/master/data-review/manual/review_manual.md)",
        "null": "If not applicable use: null",
        "todo": "If a value is not yet available, use: todo",
    },
}

# create oem template with empty lists in keys: sources, temporal, licenses, contributers, resources
oem151_empty_lists = deepcopy(oem151)
keys_to_empty = ["sources", "temporal", "licenses", "contributors", "resources"]
for keys in keys_to_empty:
    oem151_empty_lists[keys] = []

# make copy of empty oem151 template
v151_temp_backfill_clean = deepcopy(oem151_empty_lists)

# prepare oem151 keys and dicts for later reference during backfill

context_keys = list(oem151["context"].keys())
spatial_keys = list(oem151["spatial"].keys())
review_keys = list(oem151["review"].keys())

contributer_dict = deepcopy(oem151["contributors"][0])

licenses_keys = list(oem151["licenses"][0].keys())
licenses_dict = deepcopy(oem151["licenses"][0])

temporal_dict = deepcopy(oem151["temporal"])
temporal_dict["timeseries"] = [temporal_dict["timeseries"][0]]
temporal_keys = list(oem151["temporal"].keys())
ts_keys = list(oem151["temporal"]["timeseries"][0].keys())

sources_dict = deepcopy(oem151["sources"][0])
sources_licenses_keys = list(oem151["sources"][0]["licenses"][0])
sources_licenses_dict = deepcopy(oem151["sources"][0]["licenses"][0])

resources_dict = deepcopy(oem151["resources"][0])
resources_dict["schema"]["fields"] = []
resources_schema_keys = list(oem151["resources"][0]["schema"])
resources_dialect_dict = deepcopy(oem151["resources"][0]["dialect"])
resources_schema_fields = deepcopy(oem151["resources"][0]["schema"]["fields"][0])

# backfill oem141 to oem151 for all json_files in ./JSON/v141
for json_file in all_json_files_paths:

    # save filename for output
    filename = json_file.split("/")[-1].split(".")[0]

    # open oem141 files
    with open(json_file, encoding="utf-8") as json_file:
        v141_file = json.load(json_file)

    # create a clean and empty 'v151_temp_backfill' for each processed oem141 JSON
    v151_temp_backfill = deepcopy(v151_temp_backfill_clean)

    # start backfill
    # Note: '_comment' and 'metaMetadata' are not processed, as new oem151 information is provided.
    # backfill key-values on 1st level.
    first_level_keys = [
        "name",
        "title",
        "id",
        "description",
        "language",
        "keywords",
        "publicationDate",
    ]

    for key in first_level_keys:
        if key in v141_file and v141_file[key] != "":
            v151_temp_backfill[key] = v141_file[key]
        else:
            print(
                f"The key:[{key}] is not present in oemetadata v141, despite being declared in the OEM v141 template "
            )

    # backfill review keys
    for key in review_keys:
        if key in v141_file["review"].keys() and v141_file["review"][key] != "":
            v151_temp_backfill["review"][key] = v141_file["review"][key]

    # backfill context keys
    for key in context_keys:
        if key in v141_file["context"].keys() and v141_file["context"][key] != "":
            v151_temp_backfill["context"][key] = v141_file["context"][key]

    # backfill spatial keys
    for key in spatial_keys:
        if key in v141_file["spatial"].keys() and v141_file["spatial"][key] != "":
            v151_temp_backfill["spatial"][key] = v141_file["spatial"][key]

    # backfill contributors
    for index, dict in enumerate(v141_file["contributors"]):
        # add own contributer_dict for each contributer
        v151_temp_backfill["contributors"].append(deepcopy(contributer_dict))
        for key, value in dict.items():
            if v141_file["contributors"][index][key] != "":
                v151_temp_backfill["contributors"][index][key] = value

    # backfill licenses
    for index, dict in enumerate(v141_file["licenses"]):
        # add own licenses_dict for each licenses
        v151_temp_backfill["licenses"].append(deepcopy(licenses_dict))
        for key, value in dict.items():
            if v141_file["licenses"][index][key] != "":
                v151_temp_backfill["licenses"][index][key] = value

    # backfill temporal
    # Note: in oem141 only 1 temporal_dict should exist, as the key 'temporal' is just dict and not list, as in oem151.
    # add own temporal_dict for each temporal.
    v151_temp_backfill["temporal"] = temporal_dict

    for key in temporal_keys:
        if key == "timeseries":
            for timeseries_key in ts_keys:
                if v141_file["temporal"][key] is not None and timeseries_key in list(
                    v141_file["temporal"]["timeseries"].keys()
                ):
                    v151_temp_backfill["temporal"][key][0][timeseries_key] = v141_file[
                        "temporal"
                    ][key][timeseries_key]
                else:
                    print(f"{timeseries_key} is not used and filled in oem141")

        elif key in v141_file["temporal"].keys():
            if v141_file["temporal"][key] != "":
                v151_temp_backfill["temporal"][key] = v141_file["temporal"][key]

    # backfill sources
    for index, dict in enumerate(v141_file["sources"]):
        # add own sources_dict for each source
        v151_temp_backfill["sources"].append(deepcopy(sources_dict))
        for key, value in dict.items():
            if key == "licenses" and v141_file["sources"][index]["licenses"]:
                for src_lc_key in sources_licenses_keys:
                    if (
                        src_lc_key in list(v141_file["sources"][index]["licenses"][0])
                        and v141_file["sources"][index]["licenses"][0][src_lc_key] != ""
                    ):
                        v151_temp_backfill["sources"][index][key][0][
                            src_lc_key
                        ] = value[0][src_lc_key]

            elif key == "licenses" and not v141_file["sources"][index]["licenses"]:
                v151_temp_backfill["sources"][index][key][0] = sources_licenses_dict

            elif v141_file["sources"][index][key] != "":
                v151_temp_backfill["sources"][index][key] = value

    # backfill resources
    for index, dict in enumerate(v141_file["resources"]):
        # add own resources_dict for each resource
        v151_temp_backfill["resources"].append(deepcopy(resources_dict))
        for key, value in dict.items():
            if key == "schema":
                if "primaryKey" in value:
                    v151_temp_backfill["resources"][index][key][
                        "primaryKey"
                    ] = v141_file["resources"][index][key]["primaryKey"]
                if "foreignKeys" in value:
                    v151_temp_backfill["resources"][index][key][
                        "foreignKeys"
                    ] = v141_file["resources"][index][key]["foreignKeys"]

                if "fields" in value:
                    for index_fields, dict_fields in enumerate(
                        v141_file["resources"][index]["schema"]["fields"]
                    ):
                        v151_temp_backfill["resources"][index]["schema"][
                            "fields"
                        ].append(deepcopy(resources_schema_fields))
                        for key_fields, value_fields in dict_fields.items():
                            v151_temp_backfill["resources"][index]["schema"]["fields"][
                                index_fields
                            ][key_fields] = value_fields

            elif key == "dialect":
                if "delimiter" in value:
                    v151_temp_backfill["resources"][index][key]["delimiter"] = value[
                        "delimiter"
                    ]
                elif "decimalSeparator" in value:
                    v151_temp_backfill["resources"][index][key][
                        "decimalSeparator"
                    ] = value["decimalSeparator"]

            elif v141_file["resources"][index][key] != "":
                v151_temp_backfill["resources"][index][key] = value

    # save backfilled oem151 template to json in ./JSON/v151
    with open(
        f"./JSON/v151/{filename}.metadata_oem151.json", "w", encoding="utf-8"
    ) as outfile:
        json.dump(v151_temp_backfill, outfile, indent=4, ensure_ascii=False)



from omi.dialects.base.dialect import Dialect
from omi.dialects.base.register import register
from omi.dialects.oep.compiler import JSONCompiler
from omi.dialects.oep.compiler import JSONCompilerOEM15
from omi.dialects.oep.parser import JSONParser_1_3
from omi.dialects.oep.parser import JSONParser_1_4
from omi.dialects.oep.parser import JSONParser_1_5
from omi.dialects.oep.renderer import JSONRenderer
from omi.oem_structures import oem_v15


@register("oep-v1.3")
class OEP_V_1_3_Dialect(Dialect):
    _parser = JSONParser_1_3
    _compiler = None
    _renderer = JSONRenderer


@register("oep-v1.4")
class OEP_V_1_4_Dialect(Dialect):
    _parser = JSONParser_1_4
    _compiler = JSONCompiler
    _renderer = JSONRenderer


@register("oep-v1.5")
class OEP_V_1_5_Dialect(Dialect):
    _parser = JSONParser_1_5
    _compiler = JSONCompilerOEM15
    _renderer = JSONRenderer



import json
from unittest import SkipTest
from unittest import TestCase

from omi.dialects.oep import OEP_V_1_3_Dialect
from omi.dialects.oep import OEP_V_1_4_Dialect
from omi.dialects.oep import OEP_V_1_5_Dialect
from omi.dialects.oep.compiler import compile_date_or_none
from omi.dialects.oep.parser import ParserException
from omi.dialects.oep.parser import parse_date_or_none

# in the metadata, for some values we return the date,not the full datetime


class TestIssue86Datetime(TestCase):

    BAD_VALUES = [True, {}, "", "not a date", "200", "2020-30-40", "2020-01-01 WTF"]
    OK_VALUES = {
        None: None,
        2020: 2020,
        "2020": "2020",
        "2020-12": "2020-12",
        "2020-12-02": "2020-12-02",
        "2020-12-2": "2020-12-02",
        "2020-10-01T10:12:13": "2020-10-01T10:12:13",
        "2020-10-01 10:12": "2020-10-01T10:12:00",
        "2020-10-01T10:12:13+0200": "2020-10-01T10:12:13+02:00",
    }

    def roundtrip_value(self, value):
        value = parse_date_or_none(value)
        value = compile_date_or_none(value)
        return value

    def test_datetime_roundtrip(self):
        for bad_value in self.BAD_VALUES:
            self.assertRaises(ParserException, self.roundtrip_value, bad_value)
        for ok_value, exp_value in self.OK_VALUES.items():
            self.assertEqual(self.roundtrip_value(ok_value), exp_value)


class TestIssue86Metadata(TestIssue86Datetime):
    """test roundtrip in OEP_V_1_5_Dialect"""

    dialect = None
    OK_VALUES = {
        None: (None, None),
        2020: (2020, 2020),
        "2020": ("2020", "2020"),
        "2020-12": ("2020-12", "2020-12"),
        "2020-12-02": ("2020-12-02", "2020-12-02"),
        "2020-12-2": ("2020-12-02", "2020-12-02"),
        "2020-10-01T10:12:13": ("2020-10-01T10:12:13", "2020-10-01"),
        "2020-10-01 10:12": ("2020-10-01T10:12:00", "2020-10-01"),
        "2020-10-01T10:12:13+0200": ("2020-10-01T10:12:13+02:00", "2020-10-01"),
    }

    def test_datetime_roundtrip(self):
        # only actually run tests in subclasses
        if self.dialect:
            return super().test_datetime_roundtrip()

    def roundtrip_value(self, value):
        metadata_in = {"id": "test"}
        self.set_date_datetime_values(metadata_in, value)
        metadata_str = json.dumps(metadata_in)
        metadata_obj = self.dialect.parse(metadata_str)
        metadata_out = self.dialect.compile(metadata_obj)
        return self.get_date_datetime_values(metadata_out)

    def set_date_datetime_values(self, metadata, value):
        raise NotImplementedError()

    def get_date_datetime_values(self, metadata):
        raise NotImplementedError()


class TestIssue86Datetime_V_1_5(TestIssue86Metadata):
    """test roundtrip in OEP_V_1_5_Dialect"""

    dialect = OEP_V_1_5_Dialect()

    def set_date_datetime_values(self, metadata, value):
        metadata["publicationDate"] = value
        metadata["temporal"] = {"timeseries": [{"start": value}]}

    def get_date_datetime_values(self, metadata):
        v_datetime = metadata["temporal"]["timeseries"][0].get("start")
        v_date = metadata.get("publicationDate")
        return (v_datetime, v_date)


class TestIssue86Datetime_V_1_4(TestIssue86Metadata):
    """test roundtrip in OEP_V_1_4_Dialect"""

    dialect = OEP_V_1_4_Dialect()

    def set_date_datetime_values(self, metadata, value):
        metadata["publicationDate"] = value
        metadata["temporal"] = {"timeseries": {"start": value}}

    def get_date_datetime_values(self, metadata):
        v_datetime = metadata["temporal"]["timeseries"].get("start")
        v_date = metadata.get("publicationDate")
        return (v_datetime, v_date)



"""
The conversion in OMI is used to translate metadata version, usually from low to high version.

The Converter class is the base class that provides base funtionality to enable a specific version translateion.

The Translation class includes all functionality to create missing fields and supply default, none or user input values for each new field.
The translated metadata will be build and can be saved to a file by using convinience methodes provides by the converter or manually saved 
to file using OMI's dialect15.compile_and_render().

BUGS:
- oeo keys are not included
- value reference and is about got wrong name format in json
"""

import json
import logging
import pathlib
from datetime import datetime

from omi import structure
from omi.dialects import get_dialect
from omi.dialects.base.dialect import Dialect
from omi.oem_structures import oem_v15


# NOTE: Maybe change this to abstract class and implement methods in concrete child classes
class Converter:
    """
    Base Converter class for oemetadata version conversion.
    Provides basic converstion functions and helpers.
    All concrete conversion classes should inheret from this class.

    """

    def __init__(
        self,
        dielact_id: str = "oep-v1.5",  # latest version
        metadata: structure.OEPMetadata = None,
    ) -> None:
        self.dialect_id = dielact_id
        self.metadata = metadata
        self.omi_version = "OMI-v0.0.8"

    def validate_str_version_format(self):
        return NotImplementedError

    def sanitize_oem(self, oemetadata: dict) -> oem_v15.OEPMetadata:
        """
        Remove all "" or " " values. Additionaly it is possible to specify a specific
        field that will set to none. Key that got a none value will result in a json null
        that will not be visible in a json document.

        Args:
            oemetadata (oem_v15.OEPMetadata): _description_

        Returns:
            oem_v15.OEPMetadata: _description_
        """

        omi_dialect = self.detect_oemetadata_dialect(oemetadata)
        metadata = omi_dialect._parser().parse(oemetadata)
        print(type(omi_dialect))

        if (
            metadata["metaMetadata"]["metadataVersion"] == "oep-v1.5.1"
        ):  # NOTE hardcoded
            oemetadata_obj: oem_v15.OEPMetadata
            # sanitize ....
        else:
            oemetadata_obj: structure.OEPMetadata
            # sanitize ....

        return oemetadata_obj

    def format_version_string(self, version_string: str = None) -> str:
        if version_string is not None:
            version_string = version_string.lower()
            parts = version_string.split(sep="-")
            self.dialect_id = parts[0] + "-v" + parts[1][:-2]
        return self.dialect_id

    def detect_oemetadata_dialect(self, metadata=None) -> Dialect:
        if metadata is None:
            try:
                version: str = self.metadata["metaMetadata"]["metadataVersion"]
                self.dialect_id = self.format_version_string(version_string=version)
                logging.info(f"The dectected dialect is: {self.dialect_id}")
            except Exception as e:
                logging.warning(
                    {
                        "exception": f"{e}",
                        "message": f"Could not detect the dialect based on the Oemetadata json string. The key related to meta-metadata information might not be present in the input metadata json file. Fallback to the default dialect: '{self.dialect_id}'.",
                    }
                )
        else:
            try:
                version: str = metadata["metaMetadata"]["metadataVersion"]
                self.dialect_id = self.format_version_string(version_string=version)
                logging.info(f"The dectected dialect is: {self.dialect_id}")
            except Exception as e:
                logging.warning(
                    {
                        "exception": f"{e}",
                        "message": f"Could not detect the dialect based on the Oemetadata json string. The key related to meta-metadata information might not be present in the input metadata json file. Fallback to the default dialect: '{self.dialect_id}'.",
                    }
                )

        return get_dialect(identifier=self.dialect_id)

    # NOTE: Add omi version to user?
    def set_contribution(
        self,
        metadata: oem_v15.OEPMetadata,
        user: str = "OMI-v0.0.8",
        user_email: str = None,
    ) -> oem_v15.OEPMetadata:
        to_metadata = "oep-v1.5.1"  # NOTE hardcoded
        contribution = oem_v15.Contribution(
            contributor=oem_v15.Person(name=user, email=user_email),
            date=datetime.now(),
            obj="Metadata conversion",
            comment="Update metadata to "
            + to_metadata
            + " using OMIs metadata conversion tool.",
        )

        metadata = metadata.contributions.append(contribution)

        return metadata

    def convert_oemetadata():
        pass


class Metadata14To15Translation(Converter):
    """
    Converts/translates the oemetadata object generated based on the input oemetadata.json file to oemetadata-v1.5.1 structure.

    Args:
        Converter : Base converter class
    """

    def remove_temporal(
        self, metadata14: structure.OEPMetadata
    ) -> structure.OEPMetadata:
        metadata14.temporal = None
        return metadata14

    @staticmethod
    def create_subject(subject: oem_v15.Subject, name: str = "", path: str = ""):
        subject = subject()
        subject.name = name
        subject.path = path
        return subject

    @staticmethod
    def create_oeo_id(oeo_id: str = None):
        return oeo_id

    @staticmethod
    def create_oeo_context(oeo_context: str = None):
        return oeo_context

    # NOTE: need?
    @staticmethod
    def create_meta_comment_null(
        meta_comment_null: str = "If not applicable use: null",  # hardcoded
    ):
        return meta_comment_null

    @staticmethod
    def create_meta_comment_todo(
        meta_comment_todo: str = "If a value is not yet available, use: todo",  # hardcoded
    ):
        return meta_comment_todo

    @staticmethod
    def create_is_about(is_about: oem_v15.IsAbout, name: str = "", path: str = ""):
        is_about = is_about()
        is_about.name = name
        is_about.path = path
        return is_about

    @staticmethod
    def create_value_reference(
        value_reference: oem_v15.ValueReference,
        value: str = "",
        name: str = "",
        path: str = "",
    ):
        value_reference = value_reference()
        value_reference.value = value
        value_reference.name = name
        value_reference.path = path
        return value_reference

    def convert_timestamp_orientation(self, ts: str):
        new_ts = oem_v15.TimestampOrientation.create(ts)
        return new_ts

    # NOTE maybe move timeseries element from convert temporal
    def convert_timeseries(self, metadata14_temporal: structure.Temporal):
        timeseries = None
        if metadata14_temporal.ts_start and metadata14_temporal.ts_end is not None:
            timeseries = oem_v15.Timeseries(
                start=metadata14_temporal.ts_start,
                end=metadata14_temporal.ts_end,
                resolution=metadata14_temporal.ts_resolution,
                ts_orientation=self.convert_timestamp_orientation(
                    metadata14_temporal.ts_orientation.name
                ),
                aggregation=metadata14_temporal.aggregation,
            )
        return timeseries

    def convert_temporal(
        self,
        metadata14_temporal: structure.Temporal,
    ):
        temporal = None
        if metadata14_temporal is not None:
            temporal = oem_v15.Temporal(
                reference_date=metadata14_temporal.reference_date,
                timeseries_collection=[self.convert_timeseries(metadata14_temporal)],
            )  # NOTE: assume there will be a single timeseries as input because OEM-v1.4 did not support multiple timeseries elements
        return temporal

    def convert_meta_comment(
        self, metadata14_meta_comment: structure.MetaComment
    ) -> oem_v15.MetaComment:

        meta_comment = None
        if metadata14_meta_comment is not None:
            meta_comment = oem_v15.MetaComment(
                metadata_info=metadata14_meta_comment.metadata_info,
                dates=metadata14_meta_comment.dates,
                units=metadata14_meta_comment.units,
                languages=metadata14_meta_comment.languages,
                licenses=metadata14_meta_comment.licenses,
                review=metadata14_meta_comment.review,
                null=self.create_meta_comment_null(),
                todo=self.create_meta_comment_todo(),
            )

        return meta_comment

    def convert_ressources_field(self, metadata14_ressources_field: list = None):
        field: oem_v15.Field
        ressources_fields = []
        if metadata14_ressources_field is not None:
            for field in metadata14_ressources_field:
                ressources_field = oem_v15.Field(
                    name=field.name,
                    description=field.description,
                    field_type=field.type,
                    isAbout=[self.create_is_about(oem_v15.IsAbout)],
                    valueReference=[
                        self.create_value_reference(oem_v15.ValueReference)
                    ],
                    unit=field.unit,
                    resource=field.resource,
                )
                ressources_fields.append(ressources_field)
        else:
            ressources_fields = None

        return ressources_fields

    def convert_ressource(self, metadata14_ressources: list):
        ressource: oem_v15.Resource
        ressources = []
        for ressource in metadata14_ressources:
            single_ressource = oem_v15.Resource(
                name=ressource.name,
                path=ressource.path,
                profile=ressource.profile,
                resource_format=ressource.format,
                encoding=ressource.encoding,
                schema=oem_v15.Schema(
                    fields=self.convert_ressources_field(ressource.schema.fields),
                    primary_key=ressource.schema.primary_key,
                    foreign_keys=ressource.schema.foreign_keys,
                ),
                dialect=ressource.dialect,
            )
            ressources.append(single_ressource)

        return ressources

    def build_metadata15(self, metadata: structure.OEPMetadata):
        converted_metadata = oem_v15.OEPMetadata(
            name=metadata.name,
            title=metadata.title,
            identifier=metadata.identifier,
            description=metadata.description,
            subject=[
                self.create_subject(oem_v15.Subject)
            ],  # NOTE add just one dummy subject object, maybe its better to keep it empty? # add value from user input??
            languages=metadata.languages,
            keywords=metadata.keywords,
            publication_date=metadata.publication_date,
            context=metadata.context,
            spatial=metadata.spatial,
            temporal=self.convert_temporal(
                metadata.temporal
            ),  # NOTE add value from user input??
            sources=metadata.sources,
            terms_of_use=metadata.license,
            contributions=metadata.contributions,
            resources=self.convert_ressource(metadata.resources),
            databus_identifier=self.create_oeo_id(),  # NOTE add value from user input??
            databus_context=self.create_oeo_context(),  # NOTE add value from user input??
            review=metadata.review,
            comment=self.convert_meta_comment(metadata.comment),
        )

        return converted_metadata


def read_input_json(file_path: pathlib.Path = "tests/data/metadata_v14.json"):
    with open(file_path, "r", encoding="utf-8") as f:
        jsn = json.load(f)

    return jsn


def save_to_file(metadata: oem_v15.OEPMetadata, file_path: pathlib.Path):

    with open(file_path, "w", encoding="utf-8") as outfile:
        outfile.write(metadata)


def run_conversion(
    to_metadata: str, from_metadata: str, convert=Metadata14To15Translation
):
    metadata_file = read_input_json(from_metadata)
    convert = convert(metadata=metadata_file)
    dialect_input = convert.detect_oemetadata_dialect()
    metadata = dialect_input._parser().parse(metadata_file)
    convert.set_contribution(metadata)

    converted = convert.build_metadata15(metadata)

    dialect15 = get_dialect("oep-v1.5")()
    s = dialect15.compile_and_render(obj=converted)
    save_to_file(s, to_metadata)

    return s


if __name__ == "__main__":

    # Run conversion with test data
    run_conversion(
        to_metadata="1_test_results/metadata/conversion_out_oem151.json",
        from_metadata="tests/data/metadata_v14.json",
    )



from omi.dialects.oep.dialect import OEP_V_1_3_Dialect, OEP_V_1_4_Dialect, OEP_V_1_5_Dialect
from omi.dialects.oep.parser import JSONParser
from omi.dialects.oep.parser import ParserException
from omi.structure import Compilable

from metadata.latest.schema import OEMETADATA_LATEST_SCHEMA

import json


def parse_and_compile():
    inp = '{"id":"unique_id"}' #or read from json file
    dialect1_5 = OEP_V_1_5_Dialect()
    parsed = dialect1_5.parse(inp)
    print(parsed)
    parsed.identifier = "anotehr_unique_id"
    compiled = dialect1_5.compile(parsed)
    print(compiled)

def validate_oemetadata():
    parser = JSONParser()
    _input_file = "0_local_test/metadata_v15.json"

    # parser = JSONParser_1_5()
    # _input_file = "tests/data/metadata_v15.json"

    with open(_input_file, "rb") as inp:
        # file = inp.read()
        file = json.load(inp)

    # file = parser.parse_from_file(_input_file)
    # parser.validate(file, [OEMETADATA_LATEST_SCHEMA])
    return parser.is_valid(file)


# validate_oemetadata()

from omi.dialects.oep import OEP_V_1_4_Dialect, OEP_V_1_5_Dialect
from omi.dialects.oep.compiler import JSONCompiler

import pathlib

METADATA_PARSERS = [OEP_V_1_5_Dialect(), OEP_V_1_4_Dialect()]
METADATA_COMPILERS = [OEP_V_1_5_Dialect(),  OEP_V_1_4_Dialect(), JSONCompiler()]

def try_parse_metadata(inp):
    """

    Args:
        inp: string or dict or OEPMetadata

    Returns:
        Tuple[OEPMetadata or None, string or None]:
        The first component is the result of the parsing procedure or `None` if
        the parsing failed. The second component is None, if the parsing failed,
        otherwise an error message.

    Examples:

        >>> from api.actions import try_parse_metadata
        >>> result, error = try_parse_metadata('{"id":"id"}')
        >>> error is None
        True

    """

    if isinstance(inp, Compilable):
        # already parsed
        return inp, None
    elif not isinstance(inp, (str, bytes)):
        # in order to use the omi parsers, input needs to be str (or bytes)
        try:
            inp = json.dumps(inp)
        except Exception:
            return None, "Could not serialize json"

    last_err = None
    # try all the dialects
    for parser in METADATA_PARSERS:
        try:
            return parser.parse(inp), None
        except ParserException as e:
            return None, str(e)
        except Exception as e:
            last_err = e
            return None, str(e)
            # APIError(f"Metadata could not be parsed: {last_err}")
            # try next dialect

    print(f"Metadata could not be parsed: {last_err}")


def try_compile_metadata(inp):
    """

    Args:
        inp: OEPMetadata

    Returns:
        Tuple[str or None, str or None]:
        The first component is the result of the compiling procedure or `None` if
        the compiling failed. The second component is None if the compiling failed,
        otherwise an error message.
    """
    last_err = None
    # try all the dialects
    for compiler in METADATA_COMPILERS:
        try:
            return compiler.compile(inp), None
        except Exception as e:
            last_err = e
            # APIError(f"Metadata could not be compiled: {last_err}")
            # try next dialect

    print(f"Metadata could not be compiled: {last_err}")


def set_table_metadata(metadata):
    metadata_oep, err = try_parse_metadata(metadata)
    if not err:
        metadata_obj, err = try_compile_metadata(metadata_oep)
    else:
        raise Exception(err)

    if not err:
        metadata_str = json.dumps(metadata_obj, ensure_ascii=False)
        return metadata_str
    else:
        raise Exception(err)


def save_json(data: json, save_at: pathlib.Path = "examples/data/output/", filename: str = "omi_processed_metadata.json"):

    pathlib.Path(save_at).mkdir(parents=True, exist_ok=True)
    with open(f"{save_at}{filename}", "w", encoding="utf-8") as fp:
        fp.write(data)


def run_oep_case():
    # _input_file = "tests/data/metadata_v15.json"
    _input_file = "examples/data/metadata_v15.json"

    with open(_input_file, "rb") as inp:
        # file = inp.read()
        file = json.load(inp)

    jsn = set_table_metadata(file)

    save_json(data = jsn)

run_oep_case()


from omi.dialects.oep.parser import ParserException
from omi.structure import Compilable

from omi.dialects.oep import OEP_V_1_4_Dialect, OEP_V_1_5_Dialect
from omi.dialects.oep.compiler import JSONCompiler

import json


# This is a list because I copied and pasted something and normally all oemeta data versions are checked - this is not relevant for you @Jann
METADATA_PARSERS = [OEP_V_1_5_Dialect()]
METADATA_COMPILERS = [OEP_V_1_5_Dialect(), JSONCompiler()]


# check 1 - is metadata parseable by OMI
def try_parse_metadata(inp):
    """

    Args:
        inp: string or dict or OEPMetadata

    Returns:
        Tuple[OEPMetadata or None, string or None]:
        The first component is the result of the parsing procedure or `None` if
        the parsing failed. The second component is None, if the parsing failed,
        otherwise an error message.

    Examples:

        >>> from api.actions import try_parse_metadata
        >>> result, error = try_parse_metadata('{"id":"id"}')
        >>> error is None
        True
 
    """

    if isinstance(inp, Compilable):
        # already parsed
        return inp, None
    elif not isinstance(inp, (str, bytes)):
        # in order to use the omi parsers, input needs to be str (or bytes)
        try:
            inp = json.dumps(inp)
        except Exception:
            return None, "Could not serialize json"

    last_err = None
    # try all the dialects
    for parser in METADATA_PARSERS:
        try:
            return parser.parse(inp), None
        except ParserException as e:
            return None, str(e)
        except Exception as e:
            last_err = e
            return None, str(e)
            # APIError(f"Metadata could not be parsed: {last_err}")
            # try next dialect

    print(f"Metadata could not be parsed: {last_err}")

# check 2 - Is metadata compilable by omi (complies with the oemetadata spec)
def try_compile_metadata(inp):
    """

    Args:
        inp: OEPMetadata - result of omi.parse(dict)

    Returns:
        Tuple[str or None, str or None]:
        The first component is the result of the compiling procedure or `None` if
        the compiling failed. The second component is None if the compiling failed,
        otherwise an error message.
    """
    last_err = None
    # try all the dialects
    for compiler in METADATA_COMPILERS:
        try:
            return compiler.compile(inp), None
        except Exception as e:
            last_err = e
            # APIError(f"Metadata could not be compiled: {last_err}")
            # try next dialect

    print(f"Metadata could not be compiled: {last_err}")

# use this instead of omis validation
# this is what is happening on the oep before the metadata is saved to persistence
def check_oemetadata_is_oep_compatible(_input_file = "examples/data/metadata_v15.json"):  

    with open(_input_file, "rb") as inp:
        # file = inp.read()
        metadata = json.load(inp)

    metadata_oep, err = try_parse_metadata(metadata)
    if not err:
        metadata_obj, err = try_compile_metadata(metadata_oep)
    else:
        raise Exception(err)

    if not err:
        metadata_str = json.dumps(metadata_obj, ensure_ascii=False)
        return metadata_str
    else:
        raise Exception(err)

